function net = gen_random(dd,nn,pp,genpot1,genpot2)

% net = gen_random(dd,nn,pp,genpot1,genpot2)
%
% generates a random graph with nn nodes where each edge appears with 
% probability pp.  Constructs an undirected graphical model with potentials
% generated by genpot1 (for single-site potentials) and genpot2 (pairwise ones).
%
% net.numnode     the number of variables in the graphical model.
% net.vars        is a shape where indices correspond to variables, sizes 
%                 are the sizes of the domains of the variables.
% net.phi1{nn,mm} is single-site potential at location nn,mm.
% net.phi2{nn,mm} is pairwise potential on edge nn-mm.
% net.graph       is adjacency matrix of undirected graph.
% net.edge        is list of edges in undirected graph.
%
% Example: the following generates complete binary graph with Gaussian weights.
% 
%       dd = 2; nn = 10;
%       genpot1 = @() exp(randn(dd,1)*.1);
%       genpot2 = @() exp(randn(dd,dd)*.5);
%       gen_random(dd,10,1,genpot1,genpot2);


vars  = shape(1:nn,dd*ones(1,nn));
phi1  = cell(nn,1);
phi2  = cell(nn,nn);
graph = zeros(nn,nn);
edge  = zeros(0,2);

% generate single-site potentials
for ii = 1:nn
  phi1{ii} = table(genpot1(),ii);
end

% generate edges and pairwise potentials
kk = 0;
for ii = 1:nn, 
  for jj = ii+1:nn,
    if rand < pp
      kk              = kk + 1;
      phi2{ii,jj}   = table(genpot2(),[ii jj]);
      phi2{jj,ii}   = phi2{ii,jj};
      graph(jj,ii)  = 1;
      graph(ii,jj)  = 1;
      edge(kk,:)      = [ii jj];
    end
  end
end

net.vars    = vars;
net.numnode = nn;
net.phi1    = phi1;
net.phi2    = phi2;
net.graph   = graph;
net.edge    = edge;

